//! Still Learning Bevy
//! This iteration's goal is to draw a ground plane (the world) and a cuboid (the drone)
//! Functionally just ripping off the 3d shapes Bevy example

use avian3d::prelude::*;
use bevy::{
    color::palettes::basic::SILVER,
    prelude::*,
    render::{
        render_asset::RenderAssetUsages,
        render_resource::{Extent3d, TextureDimension, TextureFormat},
    },
};

mod ezreset;
use ezreset::*;

fn main() {
    App::new()
        .add_plugins((
                DefaultPlugins.set(ImagePlugin::default_nearest()),
                PhysicsPlugins::default(),
                EzResetPlugin,
        ))
        .add_systems(Startup, setup)
        .add_systems(Update, drone_up)
        .run();
}


#[derive(Component)]
struct Drone;


fn setup(
    mut commands: Commands,
    mut meshes: ResMut<Assets<Mesh>>,
    mut images: ResMut<Assets<Image>>,
    mut materials: ResMut<Assets<StandardMaterial>>,
) {
    // Define material
    let debug_material = materials.add(StandardMaterial {
        base_color_texture: Some(images.add(uv_debug_texture())),
        ..default()
    });

    // Define shapes (just the "drone")
    let shapes = [
        meshes.add(Cuboid::new(1., 0.1, 1.)),
    ];

    // Spawn shapes
    for shape in shapes.into_iter() {
        commands.spawn((
                PbrBundle {
                    mesh: shape,
                    material: debug_material.clone(),
                    transform: Transform::from_xyz(0., 2., 0.),
                    ..default()
                },
                RigidBody::Dynamic,
                Collider::cuboid(1., 0.1, 1.),
                ColliderDensity(1.),
                Drone,
                EzResetable {
                    transform: GlobalTransform::from_xyz(0., 2., 0.),
                },
        ));
    }

    // Spawn light
    commands.spawn(PointLightBundle {
        point_light: PointLight {
            shadows_enabled: true,
            intensity: 10_000_000.,
            range: 100.0,
            shadow_depth_bias: 0.2,
            ..default()
        },
        transform: Transform::from_xyz(8.0, 16.0, 8.0),
        ..default()
    });

    // Spawn ground plane
    commands.spawn((
            PbrBundle {
                mesh: meshes.add(Plane3d::default().mesh().size(50.0, 50.0).subdivisions(10)),
                material: materials.add(Color::from(SILVER)),
                ..default()
            },
            RigidBody::Static,
            Collider::half_space(Vec3::Y),
    ));

    // Spawn camera
    commands.spawn(Camera3dBundle {
        transform: Transform::from_xyz(0.0, 7., 14.0).looking_at(Vec3::new(0., 1., 0.), Vec3::Y),
        ..default()
    });
}

/// Creates a colorful test pattern
fn uv_debug_texture() -> Image {
    const TEXTURE_SIZE: usize = 8;

    let mut palette: [u8; 32] = [
        255, 102, 159, 255, 255, 159, 102, 255, 236, 255, 102, 255, 121, 255, 102, 255, 102, 255,
        198, 255, 102, 198, 255, 255, 121, 102, 255, 255, 236, 102, 255, 255,
    ];

    let mut texture_data = [0; TEXTURE_SIZE * TEXTURE_SIZE * 4];
    for y in 0..TEXTURE_SIZE {
        let offset = TEXTURE_SIZE * y * 4;
        texture_data[offset..(offset + TEXTURE_SIZE * 4)].copy_from_slice(&palette);
        palette.rotate_right(4);
    }

    Image::new_fill(
        Extent3d {
            width: TEXTURE_SIZE as u32,
            height: TEXTURE_SIZE as u32,
            depth_or_array_layers: 1,
        },
        TextureDimension::D2,
        &texture_data,
        TextureFormat::Rgba8UnormSrgb,
        RenderAssetUsages::RENDER_WORLD,
    )
}

fn drone_up(
    mut query: Query<&mut ExternalImpulse, With<Drone>>,
    keyboard: Res<ButtonInput<KeyCode>>,
) {
    if keyboard.just_pressed(KeyCode::Space) {
        for mut drone in &mut query {
            drone.apply_impulse(Vec3::Y);
        }
    }
}
